//Florian Krmer 
//Projekt Stoppuhr

org 0000h
jmp Main				//Freimachen der Interrupt-Einsprungadress


org 000Bh 				//Einsprungadresse Timer0-Interrupt

ljmp ISR_Timer0

org 001Bh				//Einsprungadresse Timer1-Interrupt

ljmp ISR_Timer1


org 0100h

Main:
			

call init		  		//Initialisierung der verwendeten Register
call timer_cfg			//Timer konfigurieren

loop:


call Ausgabe
call time				//Unterprogramm zur Zeitberechnung aufrufen

call loop				//Endlosschleife


ISR_Timer0:				//Interrupt-Service-Routine Timer0

mov TL0, #0B0h			//Timer0 vorladen
mov TH0, #3Ch			//65536-50000=15536 --> 3CB0h (50ms)
mov R5, #1				//Freigabe zur Zeitberechnung
reti


ISR_Timer1:				//Interrupt-Service-Routine Timer1

mov TL1, #0F0h			//Timer1 vorladen
mov TH1, #0D8h			//65536-10000=55536 --> D8F0h (10ms)
mov R6, #1

reti

init:				//Initialisierung der verwendeten Register

mov R0, #0			//Zhler von 0-20 --> 20Hz Interruptfrequenz (50ms) *20 =1s
mov R1, #0			//Segment rechts (Sekunden Einer)
mov R2, #0			//Segment Mitte recht (Sekunden Zehner)
mov R3, #0			//Segment Mitte links (Minuten Einer)
mov R4, #0			//Segment links (Minuten Zehner)
mov R5, #0			//"Zhlfreigabe" durch den Interrupt
mov R6, #0			//Ausgabefreigabe durch den Interrupt
mov A, #0
mov P2, #0
mov P3, #0


ret

timer_cfg:				//Timer konfigurieren

setb EA					//Interrupts generell freigeben			
setb ET0				//Timer0-Interrupt freigeben
setb ET1				//Timer1-Interrupt freigeben
mov TMOD, #00010001b	//Betriebsmode festlegen (jeweils Betriebsmode1)
setb TR0				//Timer0 luft
setb TR1				//Timer1 luft

mov TL0, #0B0h			//Timer0 vorladen
mov TH0, #3Ch			//65536-50000=15536 --> 3CB0h (50ms)

mov TL1, #0F0h			//Timer1 vorladen
mov TH1, #0D8h			//65536-10000=55536 --> D8F0h (10ms)

ret

time:

cjne R5,#1d,time		//Die Zeit kann nur berechnet werden,
mov R5, #0d				//wenn der Timer0-Interrupt ausgelst wurde

inc R0					//Zhler von 0-20 --> 20Hz Interruptfrequenz (50ms) *20 =1s
cjne R0,#20d,weiter		//fals 20 nicht erreicht wurde, Unterprogramm beenden
mov R0,#0d				//nach berlauf Register wieder 0 setzen

inc R1					//Segment rechts (Sekunden Einer)
cjne R1,#10d,weiter		//Zhlt jede Sekunde 1 hoch. Unter 10 Sprung 
						//aus dem Unterprogramm 
mov R1,#0d				//Fals 10 Sekunden erreicht wurden, wieder auf 0 Rcksetzen
						//und weiter mit Sekunden Zehner fortfahren.

inc R2					//Segment Mitte rechts (Sekunden Zehner)
cjne R2,#6d, weiter		//Zhlt jede 10 Sekunden 1 hoch. Unter 6 Sprung 
						//aus dem Unterprogramm 
mov R2,#0d				//Fals 60 Sekunden erreicht wurden, wieder auf 0 Rcksetzen
						//und weiter mit Minuten Einer fortfahren.

inc R3					//Segment Mitte links (Minuten Ener)
cjne R3,#10d, weiter		//Zhlt jede 60 Sekunden 1 hoch. Unter 10 Sprung 
						//aus dem Unterprogramm
mov R3,#0d				//Fals 10 Minuten erreicht wurden, wieder auf 0 Rcksetzen 
						//und weiter mit Minuten Zehner fortfahren.

inc R4					//Segment links (Minuten Zehner)
cjne R4,#6d, weiter		//Zhlt jede 10 Minuten 1 hoch. Unter 60 Sprung 
						//aus dem Unterprogramm
mov R4,#0d				//Fals 10 Minuten erreicht wurden, wieder auf 0 Rcksetzen
//jmp Ueberlauf			//und Fehlermeldung ausgeben, da bergeloffen.

weiter:
ret						//Sprung aus dem Unterprogramm

Ausgabe:				//Ausgabe der Zeit an die 7.Segment-Anzeige

cjne R6, #1d, Ausgabe	//Es kann nur ausgegeben werden,
mov R6, #0d				//wenn der Timer1-Interrupt ausgelst wurde

mov A, R1				//Sekunden Einer in den Akku schreiben
call Seg_Codierung		//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A				//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001110b		//Freigabe der rechten 7.Segment-Anzeige
call zeit				//Zeitschleife aufrufen

mov A, R2			  	//Sekunden Zehner in den Akku schreiben
call Seg_Codierung		//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A				//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001101b		//Freigabe der mittleren rechten 7.Segment-Anzeige
call zeit				//Zeitschleife aufrufen

mov A, R3			   	//Minuten Einer in den Akku schreiben
call Seg_Codierung	   	//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A			   	//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00001011b	   	//Freigabe der mittleren linken 7.Segment-Anzeige
call zeit				//Zeitschleife aufrufen

mov A, R4			   	//Minuten Zehner in den Akku schreiben
call Seg_Codierung	   	//die Zahl wird in eine 7-Segment-Zahl umgewandelt
mov P2, A			   	//umgewandelte Zahl wird in P2 geschrieben
mov P3, #00000111b		//Freigabe der linken 7.Segment-Anzeige
call zeit			  	//Zeitschleife aufrufen


ret


Seg_Codierung:			//wandelt die berechnete Zeit (Zahl) in eine 7-Segment-Zahl	
	mov dptr, #Tab
	movc A, @A+dptr
	ret

Tab:
db 00111111b, 00000110b, 01011011b, 01001111b, 01100110b 
db 01101101b, 01111101b, 00000111b, 01111111b, 01101111b



Zeit:					//Zeitschleife

		mov A, #60d
Zeit1:	dec A
		cjne A, #00d, Zeit1
		mov A, #60d
Zeit2:	dec A
		cjne A, #00h, Zeit2
		mov A, #00d

		ret 

end
